package com.mcwl.web.controller.memberCenter;


import cn.hutool.core.bean.BeanUtil;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.R;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.memberCenter.domain.Member;
import com.mcwl.memberCenter.domain.MemberPromotion;
import com.mcwl.memberCenter.domain.Promotion;
import com.mcwl.memberCenter.domain.dto.JoinPromotionDto;
import com.mcwl.memberCenter.domain.dto.PromotionDto;
import com.mcwl.memberCenter.enums.MemberEnum;
import com.mcwl.memberCenter.enums.PromotionEnum;
import com.mcwl.memberCenter.service.MemberPromotionService;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.memberCenter.service.PromotionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.Optional;


/**
 * 促销活动
 */
@RestController
@RequestMapping("promotion")
@RequiredArgsConstructor
@Api(tags = "会员中心")
public class PromotionController {

    private final PromotionService promotionService;

    private final MemberPromotionService memberPromotionService;

    private final MemberService memberService;


    /**
     * 创建活动
     */
    @PostMapping("createPromotion")
    @ApiOperation(value = "创建活动")
    public R<Object> createPromotion(@RequestBody @Valid PromotionDto promotionDto) {

        Date startTime = promotionDto.getStartTime();
        Date endTime = promotionDto.getEndTime();
        if (startTime.after(endTime)) {
            return R.fail("活动开始时间不能大于结束时间");
        }
        Promotion promotion = new Promotion();

        BeanUtil.copyProperties(promotionDto, promotion);

        promotionService.save(promotion);

        return R.ok();
    }


    /**
     * 活动列表
     */
    @GetMapping("promotionList")
    @ApiOperation(value = "活动列表")
    public R<List<Promotion>> promotionList() {
        List<Promotion> promotionList = promotionService.lambdaQuery()
                .gt(Promotion::getEndTime, new Date())
                .list();
        return R.ok(promotionList);
    }

    /**
     * 获取当前用户参与的活动
     */
    @GetMapping("myPromotionList")
    @ApiOperation(value = "获取当前用户参与的活动")
    public R<List<MemberPromotion>> myPromotionList() {
        // 获取当前用户
        Long userId = SecurityUtils.getUserId();
        List<MemberPromotion> memberPromotionList = memberPromotionService.lambdaQuery()
                .eq(MemberPromotion::getUserId, userId)
                .list();
        return R.ok(memberPromotionList);
    }

    /**
     * 参与活动
     */
    @PostMapping("joinPromotion")
    @ApiOperation(value = "参与活动")
    public R<Object> joinPromotion(@RequestBody @Valid JoinPromotionDto joinPromotionDto) {
        // 用户id
        Long userId = joinPromotionDto.getUserId();
        // 活动id
        Long promotionId = joinPromotionDto.getPromotionId();
        // 按活动id查询活动信息
        Promotion promotion = promotionService.getById(promotionId);

        if (!Optional.ofNullable(promotion).isPresent()) {
            return R.fail("活动不存在");
        }

        if (promotion.getStartTime().after(new Date())) {
            return R.fail("活动未开始");
        }

        // 活动是否过期
        if (promotion.getEndTime().before(new Date())) {
            return R.fail("活动已过期");
        }

        // 获取当前用户是否参与过该活动
        if (isJoinPromotion(userId, promotionId)) {
            return R.fail("您已参与过该活动");
        }

        // 是否在活动期间内订阅或续订会员
//        if (!isSubscribe(userId, promotion)) {
//            return AjaxResult.warn("请在活动期间内订阅或续期会员后参加该活动");
//        }

        Member member = memberService.getUseUserMemberByUserId(userId);

        String memberLevelIds = promotion.getMemberLevelIds();
        if (!memberLevelIds.contains(member.getMemberLevelId().toString())) {
            return R.fail("无法参与该活动，请查看活动条件");
        }
        MemberPromotion memberPromotion = getMemberPromotion(userId, promotionId);
        memberPromotionService.save(memberPromotion);


        return R.ok();
    }

//    private boolean isSubscribe(Long userId, Promotion promotion) {
//        if (promotion.getActivityType() == PromotionEnum.SUBSCRIBE) {
//            // 获取当前用户最新的订阅或续订
//            Member member = memberService.latestSubscription(userId);
//            if (!Optional.ofNullable(member).isPresent()) {
//                return false;
//            }
//            Date createTime = member.getCreateTime();
//            // 会员创建时间在活动开始时间之前，说明用户不符合“活动期间内订阅或续订会员”的条件
//            return !createTime.before(promotion.getStartTime());
//        }
//        return true;
//    }

    private boolean isJoinPromotion(Long userId, Long promotionId) {
        MemberPromotion memberPromotion = memberPromotionService.lambdaQuery()
                .eq(MemberPromotion::getUserId, userId)
                .eq(MemberPromotion::getPromotionId, promotionId)
                .one();
        return memberPromotion != null;
    }

    private static MemberPromotion getMemberPromotion(Long userId, Long promotionId) {
        MemberPromotion memberPromotion = new MemberPromotion();
        memberPromotion.setUserId(userId);
        memberPromotion.setPromotionId(promotionId);
        memberPromotion.setStatus(PromotionEnum.PARTICIPATE);
        memberPromotion.setParticipationTime(new Date());
        memberPromotion.setCreateBy(SecurityUtils.getUsername());
        memberPromotion.setCreateTime(new Date());
        memberPromotion.setUpdateBy(SecurityUtils.getUsername());
        memberPromotion.setUpdateTime(new Date());
        return memberPromotion;
    }


}
